/**
 * 
 */
package jp.co.yaskawa.rtc.integratedMotion.module.motion.kinematics;

import jp.co.yaskawa.rtc.integratedMotion.module.RTMath;

/**
 * @author motohisa
 *
 */
public class Posture {

    public static enum POSTURE_INDEX {
        X, Y, Z, ALPHA, BETA, GAMMA, ELBOW,
    }
    
	public final double x;
	public final double y;
	public final double z;
	public final double alpha;
	public final double beta;
	public final double gamma;
	public final double elbow;
	
	public Posture() {
		this(0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
	}
	
	public Posture(double x, double y, double z, double alpha, double beta, double gamma, double elbow) {
		this.x = x;
		this.y = y;
		this.z = z;
		this.alpha = alpha;
		this.beta = beta;
		this.gamma = gamma;
		this.elbow = elbow;
	}
	
	public Posture(double x, double y, double z, double alpha, double beta, double gamma) {
	    this(x, y, z, alpha, beta, gamma, 0.0);
	}
	
	public Posture(Posture p) {
		this.x = p.x;
		this.y = p.y;
		this.z = p.z;
		this.alpha = p.alpha;
		this.beta = p.beta;
		this.gamma = p.gamma;
		this.elbow = p.elbow;		
	}
	
	public Posture(HgTransMatrix tmat) {
		this.x = tmat.m03;
		this.y = tmat.m13;
		this.z = tmat.m23;
		this.elbow = 0.0;

		double sb = -tmat.m20;
		double cb2 = tmat.m21 * tmat.m21 + tmat.m22 * tmat.m22;
		
		if (RTMath.fcmp(cb2, 0.0, 10) == 0) {
			this.alpha = 0.0;
			this.beta = sb * Math.PI / 2.0;
			this.gamma = this.beta >= 0.0 ? RTMath.atan3(tmat.m12, tmat.m11) : -RTMath.atan3(tmat.m12, tmat.m11);
		} else {
			this.alpha = RTMath.atan3( tmat.m21, tmat.m22 );
			this.beta =  RTMath.atan3( -tmat.m20, Math.sqrt(cb2) );
			this.gamma =  RTMath.atan3( tmat.m10, tmat.m00 );
		}
	}
	
	public Posture(double[] p) {
		if (p.length < 7) {
			System.err.println("Posture: argument too short: " + p.length);
			throw new IllegalArgumentException();
		}
		this.x = p[0];
		this.y = p[1];
		this.z = p[2];
		this.alpha = p[3];
		this.beta = p[4];
		this.gamma = p[5];
		this.elbow = p[6];
	}
	
	   public Posture set(POSTURE_INDEX idx, double value) {
	        double x = this.x;
	        double y = this.y;
	        double z = this.z;
	        double a = this.alpha;
	        double b = this.beta;
	        double g = this.gamma;
	        double e = this.elbow;
	        
	        switch(idx) {
	        case X:
	            x = value;
	            break;
	        case Y:
	            y = value;
	            break;
	        case Z:
	            z = value;
	            break;
	        case ALPHA:
	            a = value;
	            break;
	        case BETA:
	            b = value;
	            break;
	        case GAMMA:
	            g = value;
	            break;
	        case ELBOW:
	            e = value;
	            break;
	        default:
	            throw new IllegalArgumentException();
	        }
	        
	        return new Posture(x, y, z, a, b, g, e);
	    }
	
	public Posture add(POSTURE_INDEX idx, double value) {
	    double x = this.x;
	    double y = this.y;
	    double z = this.z;
	    double a = this.alpha;
	    double b = this.beta;
	    double g = this.gamma;
	    double e = this.elbow;
	    
	    switch(idx) {
	    case X:
	        x += value;
	        break;
	    case Y:
	        y += value;
	        break;
	    case Z:
	        z += value;
	        break;
	    case ALPHA:
	        a += value;
	        break;
	    case BETA:
	        b += value;
	        break;
	    case GAMMA:
	        g += value;
	        break;
	    case ELBOW:
	        e += value;
	        break;
	    default:
	        throw new IllegalArgumentException();
	    }
	    
	    return new Posture(x, y, z, a, b, g, e);
	}
	
	public double get(POSTURE_INDEX idx) {
       switch(idx) {
        case X:
            return x;
        case Y:
            return y;
        case Z:
            return z;
        case ALPHA:
            return alpha;
        case BETA:
            return beta;
        case GAMMA:
            return gamma;
        case ELBOW:
            return elbow;
        default:
            throw new IllegalArgumentException();
        }
	}
	
	public Posture add(Posture posture) {
	    return new Posture(
	            this.x + posture.x,
	            this.y + posture.y,
	            this.z + posture.z,
	            this.alpha + posture.alpha,
	            this.beta + posture.beta,
	            this.gamma + posture.gamma,
	            this.elbow + posture.elbow
	            );
	}
	
   public Posture sub(Posture posture) {
        return new Posture(
                this.x - posture.x,
                this.y - posture.y,
                this.z - posture.z,
                this.alpha - posture.alpha,
                this.beta - posture.beta,
                this.gamma - posture.gamma,
                this.elbow - posture.elbow
                );
    }
	
	public Posture div(int n) {
	    return new Posture(
	            this.x / n,
	            this.y / n,
	            this.z / n,
	            this.alpha / n,
	            this.beta / n,
	            this.gamma /n,
	            this.elbow / n
	            );
	}
	
	public double[] toArray() {
	    double[] array = new double[POSTURE_INDEX.values().length];
	    array[0] = this.x;
	    array[1] = this.y;
	    array[2] = this.z;
	    array[3] = this.alpha;
	    array[4] = this.beta;
	    array[5] = this.gamma;
	    array[6] = this.elbow;
	    
	    return array;
	}
	
}
